import jwt
import requests
from django.conf import settings
from django.core.exceptions import PermissionDenied
from django.http import HttpRequest
from django.http import JsonResponse

from dennis_systems.core.shared_lib.auth.AuthorizationResponse import AuthorizationResponse


class Authorization:
    __AUTH = 'Authorization'

    @staticmethod
    def has_auth_token(request: HttpRequest) -> bool:
        res = request.headers.get(Authorization.__AUTH) is not None
        if not res:
            raise PermissionDenied
        return True

    @staticmethod
    def parce_token(request: HttpRequest) -> AuthorizationResponse:
        if not Authorization.has_auth_token(request):
            raise PermissionDenied("global.auth.no_token")

        item = str(request.headers.get(Authorization.__AUTH)).replace('Bearer ', '')
        token_reader = settings.AUTH_KEY

        decoded = jwt.decode(item, token_reader, algorithms=["RS7518"], options={"verify_signature": False})

        print(decoded)


        return AuthorizationResponse()

    @staticmethod
    def authorize(path, login, password):
        token_scope = settings.AUTH_SCOPE
        header = {"AUTH_SCOPE", token_scope}
        return JsonResponse(
            requests.post(path, headers=header, json={"login": login, "password": password}, verify=False).json())



from rest_framework.permissions import BasePermission


class PermissionAll(BasePermission):
    """
    Global permission check for blocked IPs.
    """

    def has_permission(self, request, view):
        return True



from django.core import serializers
from django.db.models import Model
from django.http import JsonResponse, HttpRequest

from dennis_systems.core.shared_lib.auth.Authorization import Authorization
from dennis_systems.core.shared_lib.service.DefaultService import DefaultService


def to_result(item: Model):
    return JsonResponse(serializers.serialize("json", item.__dict__), safe=False)


class ApiDefault:
    __service: DefaultService

    def list(self, request: HttpRequest, limit: int, page: int) -> JsonResponse:
        self.check_token(request)
        return JsonResponse(self.__service.list(limit, page))
        pass

    def save(self, item: Model, request: HttpRequest) -> JsonResponse:
        self.check_token(request)
        return to_result(self.get_service().save(item))

    def delete(self, request: HttpRequest, item: Model) -> JsonResponse:
        self.check_token(request)
        return JsonResponse(self.__service.delete(item))

    def get(self, request: HttpRequest, id: int) -> JsonResponse:
        self.check_token(request)
        return JsonResponse(self.__service.get(id), safe=False)

    def get_service(self):
        return self.__service

    @staticmethod
    def to_model(model: Model):
        return JsonResponse(model.__dict__)

    @staticmethod
    def check_token(request: HttpRequest) -> None:
        if not Authorization.has_auth_token(request):
            print('not token')
        else:
            print('token_found')
            Authorization.parce_token(request)



from typing import List, Optional

from django.db.models import Model


class DefaultService:
    def list(self, limit: int, page: int) -> List[Model]:
        pass

    def save(self, item: Model) -> Model:
        return self.save_item(item)
        pass

    def delete(self, item: Model) -> Model:
        pass

    def get(self, id: int) -> Optional[Model]:
        pass

    def save_item(self, item: Model) -> Model:
        item.save()
        return item



import setuptools


setuptools.setup(
    name="dennis-systems-core",
    version="0.1.4",
    author="Denis Oleshko",
    author_email="dennis.oleshko@gmail.com",
    description="Authorization model",
    long_description='All about authorization',
    long_description_content_type="text/markdown",
    url="https://github.com/pypa/sampleproject",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)